@@ -3,14 +3,27 @@
 # convenient Python types.
 
 import ephem._libastro as _libastro
+import re
 from datetime import datetime as _datetime
 from datetime import timedelta as _timedelta
 from datetime import tzinfo as _tzinfo
-from math import pi
+from math import acos, cos, pi, sin
 from time import localtime as _localtime
 
-__version__ = '3.7.7.1'
+__version__ = '4.1.3'
 
+# As a favor, compile a regular expression that our C library would
+# really rather not compile for itself.
+
+_libastro._scansexa_split = re.compile(r'''
+    \s*:\s*         # A colon optionally surrounded by whitespace,
+    |               # or,
+    (?<!^)\s+(?!$)  # whitespace not at the start or end of the string.
+''', re.X).split
+
+# Various constants.
+
+tau = 6.283185307179586476925287
 twopi = pi * 2.
 halfpi = pi / 2.
 quarterpi = pi / 4.
@@ -32,6 +45,9 @@ B1900 = 2415020.3135 - _libastro.MJD0
 B1950 = 2433282.4235 - _libastro.MJD0
 J2000 = _libastro.J2000
 
+_slightly_less_than_zero = -1e-15
+_slightly_more_than_pi = pi + 1e-15
+
 # We make available several basic types from _libastro.
 
 Angle = _libastro.Angle
@@ -61,6 +77,7 @@ readdb = _libastro.readdb
 readtle = _libastro.readtle
 constellation = _libastro.constellation
 separation = _libastro.separation
+unrefract = _libastro.unrefract
 now = _libastro.now
 
 millennium_atlas = _libastro.millennium_atlas
@@ -86,6 +103,11 @@ Jupiter = _libastro.Jupiter
 Saturn = _libastro.Saturn
 Moon = _libastro.Moon
 
+# Angles.
+
+def _plusminus_pi(angle):
+    return (angle - pi) % tau - pi
+
 # Newton's method.
 
 def newton(f, x0, x1, precision=default_newton_precision):
@@ -239,6 +261,9 @@ class NeverUpError(CircumpolarError): pass
 class AlwaysUpError(CircumpolarError): pass
 
 def describe_riset_search(method):
+    if method.__doc__ is None:
+        return method
+
     method.__doc__ += """, returning its date.
 
     The search starts at the `date` of this `Observer` and is limited to
@@ -261,8 +286,8 @@ class Observer(_libastro.Observer):
     set its attributes once you have created it.  Defaults:
 
     `date` - the moment the `Observer` is created
-    `lat` - zero degrees latitude
-    `lon` - zero degrees longitude
+    `lat` - zero latitude
+    `lon` - zero longitude
     `elevation` - 0 meters above sea level
     `horizon` - 0 degrees
     `epoch` - J2000
@@ -285,6 +310,8 @@ class Observer(_libastro.Observer):
         o.pressure = self.pressure
         return o
 
+    __copy__ = copy
+
     def __repr__(self):
         """Return a useful textual representation of this Observer."""
         return ('<ephem.Observer date=%r epoch=%r'
@@ -396,9 +423,27 @@ class Observer(_libastro.Observer):
                                 ' above the horizon at latitude %s'
                                 % (declination, self.lat))
 
-    def _riset_helper(self, body, start, use_center, rising, previous):
-        """Internal function for finding risings and settings."""
+    @describe_riset_search
+    def previous_rising(self, body, start=None, use_center=False):
+        """Search for the given body's previous rising"""
+        return self._find_rise_or_set(body, start, use_center, -1, True)
 
+    @describe_riset_search
+    def previous_setting(self, body, start=None, use_center=False):
+        """Search for the given body's previous setting"""
+        return self._find_rise_or_set(body, start, use_center, -1, False)
+
+    @describe_riset_search
+    def next_rising(self, body, start=None, use_center=False):
+        """Search for the given body's next rising"""
+        return self._find_rise_or_set(body, start, use_center, +1, True)
+
+    @describe_riset_search
+    def next_setting(self, body, start=None, use_center=False):
+        """Search for the given body's next setting"""
+        return self._find_rise_or_set(body, start, use_center, +1, False)
+
+    def _find_rise_or_set(self, body, start, use_center, direction, do_rising):
         if isinstance(body, EarthSatellite):
             raise TypeError(
                 'the rising and settings methods do not'
@@ -406,110 +451,80 @@ class Observer(_libastro.Observer):
                 ' please use the higher-resolution next_pass() method'
                 )
 
-        def visit_transit():
-            d = (previous and self._previous_transit(body)
-                 or self._next_transit(body)) # if-then
-            if body.alt + body.radius * use_radius - self.horizon <= 0:
-                raise NeverUpError('%r transits below the horizon at %s'
-                                   % (body.name, d))
-            return d
-
-        def visit_antitransit():
-            d = (previous and self._previous_antitransit(body)
-                 or self._next_antitransit(body)) # if-then
-            if body.alt + body.radius * use_radius - self.horizon >= 0:
-                raise AlwaysUpError('%r is still above the horizon at %s'
-                                    % (body.name, d))
-            return d
-
-        # Determine whether we should offset the result for the radius
-        # of the object being measured, or instead pretend that rising
-        # and setting happens when its center crosses the horizon.
-        if use_center:
-            use_radius = 0.0
-        else:
-            use_radius = 1.0
-
-        # Save self.date so that we can restore it before returning.
+        original_pressure = self.pressure
         original_date = self.date
-
-        # Start slightly to one side of the start date, to prevent
-        # repeated calls from returning the same solution over and over.
-        if start is not None:
-            self.date = start
-        if previous:
-            self.date -= default_newton_precision
-        else:
-            self.date += default_newton_precision
-
-        # Take a big leap towards the solution, then Newton takes us home.
-        body.compute(self)
-        heading_downward = (rising == previous) # "==" is inverted "xor"
-        if heading_downward:
-            on_lower_cusp = (body.alt + body.radius * use_radius
-                             - self.horizon > tiny)
-        else:
-            on_lower_cusp = (body.alt + body.radius * use_radius
-                             - self.horizon < - tiny)
-
-        az = body.az
-        on_right_side_of_sky = ((rising == (az < pi)) # inverted "xor"
-                                or (az < tiny
-                                    or pi - tiny < az < pi + tiny
-                                    or twopi - tiny < az))
-
-        def f(d):
-            self.date = d
-            body.compute(self)
-            return body.alt + body.radius * use_radius - self.horizon
-
         try:
-            if on_lower_cusp and on_right_side_of_sky:
-                d0 = self.date
-            elif heading_downward:
-                d0 = visit_transit()
-            else:
-                d0 = visit_antitransit()
-            if heading_downward:
-                d1 = visit_antitransit()
-            else:
-                d1 = visit_transit()
+            self.pressure = 0.0  # otherwise geometry doesn't work
+            if start is not None:
+                self.date = start
+            prev_ha = None
+            while True:
+                body.compute(self)
+                horizon = self.horizon
+                if not use_center:
+                    horizon -= body.radius
+                if original_pressure:
+                    horizon = unrefract(original_pressure, self.temp, horizon)
+                abs_target_ha = self._target_hour_angle(body, horizon)
+                if do_rising:
+                    target_ha = - abs_target_ha  # rises in east (az 0-180)
+                else:
+                    target_ha = abs_target_ha    # sets in west (az 180-360)
+                ha = body.ha
+                difference = target_ha - ha
+                if prev_ha is None:
+                    difference %= tau  # force angle to be positive
+                    if direction < 0:
+                        difference -= tau
+                    bump = difference / tau
+                    if abs(bump) < default_newton_precision:
+                        # Already at target event: move forward to next one.
+                        bump += direction
+                else:
+                    difference = _plusminus_pi(difference)
+                    bump = difference / tau
+                if abs(bump) < default_newton_precision:
+                    break
+                self.date += bump
+                prev_ha = ha
 
-            d = (d0 + d1) / 2.
-            result = Date(newton(f, d, d + minute))
-            return result
+            if abs_target_ha == _slightly_more_than_pi:
+                raise AlwaysUpError('%r is above the horizon at %s'
+                                    % (body.name, self.date))
+
+            if abs_target_ha == _slightly_less_than_zero:
+                raise NeverUpError('%r is below the horizon at %s'
+                                   % (body.name, self.date))
+
+            return self.date
         finally:
+            if self.pressure != original_pressure:
+                self.pressure = original_pressure
+                body.compute(self)
             self.date = original_date
 
-    @describe_riset_search
-    def previous_rising(self, body, start=None, use_center=False):
-        """Search for the given body's previous rising"""
-        return self._riset_helper(body, start, use_center, True, True)
+    def _target_hour_angle(self, body, alt):
+        lat = self.lat
+        dec = body.dec
+        arg = (sin(alt) - sin(lat) * sin(dec)) / (cos(lat) * cos(dec))
 
-    @describe_riset_search
-    def previous_setting(self, body, start=None, use_center=False):
-        """Search for the given body's previous setting"""
-        return self._riset_helper(body, start, use_center, False, True)
+        if arg < -1.0:
+            return _slightly_more_than_pi
+        elif arg > 1.0:
+            return _slightly_less_than_zero
 
-    @describe_riset_search
-    def next_rising(self, body, start=None, use_center=False):
-        """Search for the given body's next rising"""
-        return self._riset_helper(body, start, use_center, True, False)
-
-    @describe_riset_search
-    def next_setting(self, body, start=None, use_center=False):
-        """Search for the given body's next setting"""
-        return self._riset_helper(body, start, use_center, False, False)
+        return acos(arg)
 
     def next_pass(self, body, singlepass=True):
         """Return the next rising, culmination, and setting of a satellite.
-        
-        If singlepass is True, return next consecutive set of
-            (rising, culmination, setting).
-        If singlepass is False, return 
-            (next_rising, next_culmination, next_setting)
-        """
 
+        If singlepass is True, return next consecutive set of
+        ``(rising, culmination, setting)``.
+
+        If singlepass is False, return
+        ``(next_rising, next_culmination, next_setting)``.
+
+        """
         if not isinstance(body, EarthSatellite):
             raise TypeError(
                 'the next_pass() method is only for use with'
@@ -519,7 +534,7 @@ class Observer(_libastro.Observer):
         result = _libastro._next_pass(self, body)
         # _libastro behavior is singlepass=False
         if ((not singlepass)
-                or (None in result) 
+                or (None in result)
                 or (result[4] >= result[0])):
             return result
         # retry starting just before next_rising
@@ -532,7 +547,7 @@ class Observer(_libastro.Observer):
         if result[0] <= result[2] <= result[4]:
             return result
         raise ValueError("this software is having trouble with those satellite parameters")
-        
+
 
 del describe_riset_search
 
